@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-import math
+import json
 import os
 import re
 import sys
@@ -11,6 +11,16 @@ from urllib.parse import urlparse
 
 import requests
 from PIL import Image
+from rich.progress import (
+    BarColumn,
+    MofNCompleteColumn,
+    Progress,
+    SpinnerColumn,
+    TaskProgressColumn,
+    TextColumn,
+    TimeElapsedColumn,
+    TimeRemainingColumn,
+)
 
 HEADERS = {
     "User-Agent": (
@@ -70,6 +80,8 @@ class GetJump:
         only_first: bool = False,
         username: str | None = None,
         password: str | None = None,
+        save_metadata: bool = False,
+        print_log: bool = False,
     ) -> tuple[str | None, str, bool]:
         self.__check_url(url)
         self.login(url, username, password)
@@ -94,18 +106,25 @@ class GetJump:
 
         save_dir = os.path.join(save_path, series_title, title)
         if os.path.exists(save_dir) and not overwrite:
-            print("already existed! (to overwrite, use `-o`)", file=sys.stderr)
+            if print_log:
+                print("already existed! (to overwrite, use `-o`)", file=sys.stderr)
             return nxt, save_dir, False
         os.makedirs(save_dir, exist_ok=True)
 
         if not j["isPublic"] and not j["hasPurchased"]:
             warnings.warn(title, NeedPurchase, stacklevel=1)
-            print(j["isPublic"], j["hasPurchased"])
+            if print_log:
+                print(j["isPublic"], j["hasPurchased"])
             return nxt, save_dir, False
         else:
             pages: list[Page] = [p for p in j["pageStructure"]["pages"] if "src" in p]
 
-        self.__save_images(pages, save_dir, only_first)
+        if save_metadata:
+            print(
+                json.dumps(res.json(), indent=4),
+                file=open(os.path.join(save_dir, "metadata.json"), "w"),
+            )
+        self.__save_images(pages, save_dir, only_first, print_log=print_log)
 
         return nxt, save_dir, True
 
@@ -150,7 +169,8 @@ class GetJump:
             self._loggedin_hosts.append(base_url)
         else:
             raise ValueError(
-                f"Maybe login (to: {login_url}) is failed (code: {status_code}). Is given information correct?"
+                f"Maybe login (to: {login_url}) is failed (code: {status_code}). "
+                "Is given information correct?"
             )
         return res
 
@@ -170,27 +190,53 @@ class GetJump:
         return nxt + ".json" if type(nxt) is str else nxt
 
     def __save_images(
-        self, pages: list[Page], save_dir: str, only_first: bool = False
+        self,
+        pages: list[Page],
+        save_dir: str,
+        only_first: bool = False,
+        print_log: bool = False,
     ) -> None:
-        imgs: list[Image.Image] = []
-        for page in pages:
-            img = self.__get_image(page["src"])
-            imgs.append(img)
-            if only_first:
-                break
-
-        len_page_digit = len(str(len(imgs)))
-        for idx, img in enumerate(imgs):
-            save_img_path = os.path.join(
-                save_dir, f"%0{len_page_digit}d" % idx + ".jpg"
+        progress = Progress(
+            SpinnerColumn(),
+            TextColumn("{task.description}"),
+            BarColumn(),
+            TaskProgressColumn(),
+            TextColumn("("),
+            MofNCompleteColumn(),
+            TextColumn("pages"),
+            TextColumn(")"),
+            TextColumn("remain:"),
+            TimeRemainingColumn(),
+            TextColumn("spent:"),
+            TimeElapsedColumn(),
+            disable=not print_log,
+        )
+        with progress:
+            imgs: list[Image.Image] = []
+            task_dl = progress.add_task(
+                "[red]Downloading...", total=1 if only_first else len(pages)
             )
-            img.save(save_img_path)
+            for page in pages:
+                img = self.__get_image(page["src"])
+                imgs.append(img)
+                progress.update(task_dl, advance=1)
+                if only_first:
+                    break
+
+            len_page_digit = len(str(len(imgs)))
+            task_save = progress.add_task("[green]Saving...", total=len(imgs))
+            for idx, img in enumerate(imgs):
+                save_img_path = os.path.join(
+                    save_dir, f"%0{len_page_digit}d" % idx + ".jpg"
+                )
+                img.save(save_img_path)
+                progress.update(task_save, advance=1)
 
     def __get_image(self, image_src: str, div: int = 4, mul: int = 8) -> Image.Image:
         img = Image.open(BytesIO(requests.get(image_src).content))
         img_width, img_height = img.size
-        fixed_width = math.floor(float(img_width) / (div * mul)) * mul
-        fixed_height = math.floor(float(img_height) / (div * mul)) * mul
+        fixed_width = int(float(img_width) / (div * mul)) * mul
+        fixed_height = int(float(img_height) / (div * mul)) * mul
         buff: list[list[Image.Image]] = []
         for x in range(div):
             inbuff: list[Image.Image] = []
@@ -208,5 +254,5 @@ class GetJump:
 
         for y, inbuff in enumerate(buff):
             for x, cropped in enumerate(inbuff):
-                img.paste(cropped, (int(fixed_width * x), int(fixed_height * y)))
+                img.paste(cropped, box=(fixed_width * x, fixed_height * y))
         return img
