@@ -103,18 +103,32 @@ def parse_args(test: list[str] | None = None) -> argparse.Namespace:
         action="store_true",
         help="overwrite",
     )
+    parser.add_argument(
+        "-m",
+        "--metadata",
+        action="store_true",
+        help="save metadata as json",
+    )
     parser.add_argument(
         "-u",
         "--username",
         type=str,
+        metavar="ID",
         help="username if you want to login",
     )
     parser.add_argument(
         "-p",
         "--password",
+        metavar="PW",
         type=str,
         help="password if you want to login",
     )
+    parser.add_argument(
+        "-q",
+        "--quiet",
+        action="store_true",
+        help="disable console print",
+    )
     parser.add_argument("-V", "--version", action="version", version=__version__)
 
     if test:
@@ -129,7 +143,8 @@ def parse_args(test: list[str] | None = None) -> argparse.Namespace:
 def get_bulk(args: argparse.Namespace) -> None:
     g = GetJump()
     next_uri = args.url
-    print("get:", next_uri)
+    if not args.quiet:
+        print("get:", next_uri)
     while next_uri:
         next_uri, prev_title, ok = g.get(
             next_uri,
@@ -138,19 +153,21 @@ def get_bulk(args: argparse.Namespace) -> None:
             only_first=args.first,
             username=args.username,
             password=args.password,
+            save_metadata=args.metadata,
+            print_log=not args.quiet,
         )
-        if ok:
-            print("saved:", prev_title)
-        if next_uri is not None:
-            print("next:", next_uri)
-    else:
-        print("done.")
+        if not args.quiet:
+            if ok:
+                print("saved:", prev_title)
+            if next_uri is not None:
+                print("next:", next_uri)
 
 
 def get_one(args: argparse.Namespace) -> None:
     g = GetJump()
     next_uri = args.url
-    print("get:", next_uri)
+    if not args.quiet:
+        print("get:", next_uri)
     _, prev_title, ok = g.get(
         next_uri,
         save_path=args.savedir,
@@ -158,10 +175,11 @@ def get_one(args: argparse.Namespace) -> None:
         only_first=args.first,
         username=args.username,
         password=args.password,
+        save_metadata=args.metadata,
+        print_log=not args.quiet,
     )
     if ok:
         print("saved:", prev_title)
-    print("done.")
 
 
 def main() -> None:
@@ -173,7 +191,8 @@ def main() -> None:
         get_bulk(args)
     else:
         get_one(args)
-
+    if not args.quiet:
+        print("done.")
 
 if __name__ == "__main__":
     main()
